#!/usr/bin/env python3

import csv
import io

import tkinter as tk

import defs

FILL = tk.N + tk.S + tk.W + tk.E

class App(tk.Frame):
    def __init__(self, master=None):
        tk.Frame.__init__(self, master)
        self.cargos = defs.Cargos()
        self.column = 0
        self.row = 0
        self.grid(sticky=FILL)
        self.create_widgets()
        self.fill_unset()

    def button_command_factory(self, source_listbox, dest_listbox):
        def inner(event=None):
            for s in source_listbox.curselection()[::-1]:
                dest_listbox.insert(tk.END, source_listbox.get(s))
                source_listbox.delete(s)
        return inner

    def multi_command_factory(self, *funcs):
        def inner(*args, **kwargs):
            for f in funcs:
                f(*args, **kwargs)
        return inner

    def add_listbox(self, header, command=None):
        l = tk.Label(self, text=header)
        l.grid(column=self.column, row=self.row)
        lb = tk.Listbox(self, activestyle='none', exportselection=0,
                        height=40, selectmode=tk.EXTENDED)
        if command:
            lb.bind('<<ListboxSelect>>', command)
        lb.grid(column=self.column, row=self.row+1, rowspan=2, sticky=FILL)
        return lb

    def add_move_buttons(self, add, del_):
        cmd = self.multi_command_factory(
            self.button_command_factory(add[1], add[2]),
            self.sort_listboxes)
        btn_add = tk.Button( self, text=add[0], command=cmd)
        btn_add.grid(column=self.column, row=self.row+1, sticky=tk.S)
        cmd = self.multi_command_factory(
            self.button_command_factory(del_[1], del_[2]),
            self.sort_listboxes)
        btn_del = tk.Button(self, text=del_[0], command=cmd)
        btn_del.grid(column=self.column, row=self.row+2, sticky=tk.N)
        return btn_add, btn_del

    def create_widgets(self):
        top = self.winfo_toplevel()
        # Settings
        top.rowconfigure(0, weight=1)
        top.columnconfigure(0, weight=1)
        self.rowconfigure(1, weight=1)
        self.rowconfigure(2, weight=1)
        self.columnconfigure(0, weight=1)
        self.columnconfigure(2, weight=1)
        self.columnconfigure(4, weight=1)
        self.columnconfigure(5, weight=1)
        self.columnconfigure(7, weight=1)
        self.columnconfigure(9, weight=1)

        # Menubar
        self.menubar = tk.Menu(top)
        top['menu'] = self.menubar
        self.submenu = tk.Menu(self.menubar, tearoff=False)
        self.menubar.add_cascade(label='File', menu=self.submenu)
        self.submenu.entryconfigure(2, state=tk.DISABLED)

        # Cargo labels
        self.lb_label_allow = self.add_listbox(
            'cargo_allow', self.update_selected_ccs)
        self.column += 2
        self.lb_label_unset = self.add_listbox(
            'Unset cargos', self.update_selected_ccs)
        self.column += 2
        self.lb_label_disallow = self.add_listbox(
            'cargo_disallow', self.update_selected_ccs)
        self.column += 1

        # Cargo classes
        self.lb_cc_allow = self.add_listbox(
            'refittable_cargo_classes', self.update_selected_labels)
        self.column += 2
        self.lb_cc_unset = self.add_listbox(
            'Unset cargo classes', self.update_selected_labels)
        self.column += 2
        self.lb_cc_disallow = self.add_listbox(
            'non_refittable_cargo_classes', self.update_selected_labels)

        # Buttons have to be added after listboxes, for commands
        self.column -= 8
        self.add_move_buttons(
            ('<-', self.lb_label_unset, self.lb_label_allow),
            ('->', self.lb_label_allow, self.lb_label_unset)
        )
        self.column += 2
        self.add_move_buttons(
            ('->', self.lb_label_unset, self.lb_label_disallow),
            ('<-', self.lb_label_disallow, self.lb_label_unset)
        )
        self.column += 3
        self.add_move_buttons(
            ('<-', self.lb_cc_unset, self.lb_cc_allow),
            ('->', self.lb_cc_allow, self.lb_cc_unset)
        )
        self.column += 2
        self.add_move_buttons(
            ('->', self.lb_cc_unset, self.lb_cc_disallow),
            ('<-', self.lb_cc_disallow, self.lb_cc_unset)
        )
        #self.column += 2

        # Export buttons
        self.column -= 8
        self.row += 3
        btn_export = tk.Button(self, text='Copy to clipboard (CSV)',
                               command=self.export_csv)
        btn_export.grid(column=self.column, row=self.row)

    def export_csv(self):
        row = []
        row.append(', '.join(self.lb_label_allow.get(0, tk.END)))
        row.append(', '.join(self.lb_label_disallow.get(0, tk.END)))
        row.append(', '.join(self.cargos.cc_map[cc]['name_nml'][3:] \
                             for cc in self.lb_cc_allow.get(0, tk.END)))
        row.append(', '.join(self.cargos.cc_map[cc]['name_nml'][3:] \
                             for cc in self.lb_cc_disallow.get(0, tk.END)))
        
        f = io.StringIO()
        writer = csv.writer(f, delimiter='\t')
        writer.writerow(row)
        self.clipboard_clear()
        self.clipboard_append(f.getvalue())

    def fill_unset(self):
        for label in self.cargos.cargo_map:
            self.lb_label_unset.insert(tk.END, label)
        for cc in self.cargos.cc_map:
            self.lb_cc_unset.insert(tk.END, cc)

    def get_selected(self, listbox):
        out = set()
        for s in listbox.curselection():
            out.add(listbox.get(s))
        return out

    def sort_listboxes(self):
        def inner(listboxes, source):
            for lb in listboxes:
                listed = set(lb.get(0, tk.END))
                lb.delete(0, tk.END)
                for elem in source:
                    if elem in listed:
                        lb.insert(tk.END, elem)
        label_boxes = [self.lb_label_allow, self.lb_label_unset,
                       self.lb_label_disallow]
        inner(label_boxes, self.cargos.cargo_map)
        cc_boxes = [self.lb_cc_allow, self.lb_cc_unset, self.lb_cc_disallow]
        inner(cc_boxes, self.cargos.cc_map)

    def selected_labels(self):
        out = set()
        out.update(self.get_selected(self.lb_label_allow))
        out.update(self.get_selected(self.lb_label_unset))
        out.update(self.get_selected(self.lb_label_disallow))
        return out

    def selected_ccs(self):
        out = set()
        out.update(self.get_selected(self.lb_cc_allow))
        out.update(self.get_selected(self.lb_cc_unset))
        out.update(self.get_selected(self.lb_cc_disallow))
        return out

    def update_selected_ccs(self, event):
        self.lb_cc_allow.selection_clear(0, tk.END)
        self.lb_cc_unset.selection_clear(0, tk.END)
        self.lb_cc_disallow.selection_clear(0, tk.END)
        cc_listboxes = [self.lb_cc_allow, self.lb_cc_unset, self.lb_cc_disallow]
        for label in self.selected_labels():
            for cc in self.cargos.cargo_map[label]:
                for cc_lb in cc_listboxes:
                    for cci in range(0, cc_lb.size()+1):
                        if cc_lb.get(cci) == cc:
                            cc_lb.selection_set(cci)

    def update_selected_labels(self, event):
        self.lb_label_allow.selection_clear(0, tk.END)
        self.lb_label_unset.selection_clear(0, tk.END)
        self.lb_label_disallow.selection_clear(0, tk.END)
        label_listboxes = [self.lb_label_allow, self.lb_label_unset,
                           self.lb_label_disallow]
        for cargo_class in self.selected_ccs():
            for label in self.cargos.cc_map[cargo_class]['cargos']:
                for label_lb in label_listboxes:
                    for lbli in range(0, label_lb.size()+1):
                        if label_lb.get(lbli) == label:
                            label_lb.selection_set(lbli)


app = App()
app.mainloop()
